;Adaptation purebasic : Comtois le 25/02/06
;
;/********************************************************/
;/* AABB-triangle overlap test code                      */
;/* by Tomas Akenine-Mller                              */
;/* Function: int triBoxOverlap(float boxcenter[3],      */
;/*          float boxhalfsize[3],float triverts[3][3]); */
;/* History:                                             */
;/*   2001-03-05: released the code in its first version */
;/*   2001-06-18: changed the order of the tests, faster */
;/*                                                      */
;/* Acknowledgement: Many thanks To Pierre Terdiman For  */
;/* suggestions And discussions on how To optimize code. */
;/* Thanks To David Hunt For finding a ">="-bug!         */
;/********************************************************/

#X=0

#Y=1

#Z=2

XIncludeFile "Include\Maths.Pbi"

Macro FINDMINMAX(x0,x1,x2,min,max) 

  min = x0
  
  max = x0   

  If (x1<min) : min=x1 : EndIf

  If (x1>max) : max=x1 : EndIf

  If (x2<min) : min=x2 : EndIf

  If (x2>max) : max=x2 : EndIf

EndMacro

Procedure planeBoxOverlap(*normal.s_Vecteur, *vert.s_Vecteur, *maxbox.s_Vecteur)	
 
  Define.f v
  
  Define.s_Vecteur vmin, vmax

  v=*vert\x		

  If *normal\x>0.0

    vmin\x=-*maxbox\x - v

    vmax\x= *maxbox\x - v

  Else

    vmin\x= *maxbox\x - v

    vmax\x=-*maxbox\x - v

  EndIf

  v=*vert\y		

  If *normal\y>0.0

    vmin\y=-*maxbox\y - v

    vmax\y= *maxbox\y - v

  Else

    vmin\y= *maxbox\y - v

    vmax\y=-*maxbox\y - v

  EndIf
    
  v=*vert\z	

  If *normal\z>0.0

    vmin\z=-*maxbox\z - v

    vmax\z= *maxbox\z - v

  Else

    vmin\z= *maxbox\z - v

    vmax\z=-*maxbox\z - v

  EndIf

  If ProduitScalaire(*normal,vmin)>0.0  : ProcedureReturn #False : EndIf

  If ProduitScalaire(*normal,vmax)>=0.0 : ProcedureReturn #True : EndIf  

  ProcedureReturn #False

EndProcedure

;/*======================== X-tests ========================*/

Macro AXISTEST_X01(a, b, fa, fb)			   

	p0 = a*v0\y - b*v0\z			       	   

	p2 = a*v2\y - b*v2\z			       	   

  If p0<p2
  
    min=p0
  
    max=p2
  
  Else
  
    min=p2
  
    max=p0
  
  EndIf

	rad = fa * *boxhalfsize\y + fb * *boxhalfsize\z

	If min>rad Or max<-rad : ProcedureReturn #False : EndIf
	
EndMacro


Macro AXISTEST_X2(a, b, fa, fb)			   

	p0 = a*v0\y - b*v0\z			           

	p1 = a*v1\y - b*v1\z			       	   

  If p0<p1 
    
    min=p0
    
    max=p1
  
  Else
    
    min=p1
    
    max=p0
    
  EndIf  

	rad = fa * *boxhalfsize\y + fb * *boxhalfsize\z

	If min>rad Or max<-rad : ProcedureReturn #False : EndIf
	
EndMacro


;/*======================== Y-tests ========================*/

Macro AXISTEST_Y02(a, b, fa, fb)			   

	p0 = -a*v0\x + b*v0\z		      	   

	p2 = -a*v2\x + b*v2\z	       	       	   

  If p0<p2
    
    min=p0
    
    max=p2
  
  Else
  
    min=p2
    
    max=p0
    
  EndIf 

	rad = fa * *boxhalfsize\x + fb * *boxhalfsize\z

	If min>rad Or max<-rad : ProcedureReturn #False : EndIf

EndMacro

Macro AXISTEST_Y1(a, b, fa, fb)			   

	p0 = -a*v0\x + b*v0\z	      	   

	p1 = -a*v1\x + b*v1\z     	       	   

  If p0<p1
  
    min=p0
    
    max=p1
    
  Else
  
    min=p1
    
    max=p0
    
  EndIf  

	rad = fa * *boxhalfsize\x + fb * *boxhalfsize\z

	If min>rad Or max<-rad :  ProcedureReturn #False : EndIf

EndMacro

;/*======================== Z-tests ========================*/

Macro AXISTEST_Z12(a, b, fa, fb)			   

	p1 = a*v1\x - b*v1\y			           

	p2 = a*v2\x - b*v2\y			       	   

  If p2<p1 
    
    min=p2
    
    max=p1
  
  Else 
  
    min=p1
  
    max=p2
  
  EndIf

	rad = fa * *boxhalfsize\x + fb * *boxhalfsize\y

	If min>rad Or max<-rad : ProcedureReturn #False : EndIf
	
EndMacro


Macro AXISTEST_Z0(a, b, fa, fb)			  

	p0 = a*v0\x - b*v0\y				   

	p1 = a*v1\x - b*v1\y			          

  If p0<p1
  
    min=p0
    
    max=p1
    
  Else
  
    min=p1
    
    max=p0
    
  EndIf 

	rad = fa * *boxhalfsize\x + fb * *boxhalfsize\y  

	If min>rad Or max<-rad : ProcedureReturn #False : EndIf

EndMacro


Procedure triBoxOverlap(*boxcenter.s_Vecteur, *boxhalfsize.s_Vecteur, triverts.s_Vecteur(1))
  ;/*    use separating axis theorem To test overlap between triangle And box */

  ;/*    need To test For overlap in these directions: */

  ;/*    1) the {x,y,z}-directions (actually, since we use the AABB of the triangle */

  ;/*       we do Not even need To test these) */

  ;/*    2) normal of the triangle */

  ;/*    3) crossproduct(edge from tri, {x,y,z}-directin) */

  ;/*       this gives 3x3=9 more tests */


   Define.s_Vecteur v0, v1, v2, normal, e0, e1, e2
  
   Define.f min, max, p0, p1, p2, rad, fex, fey, fez 
  
   ;/* This is the fastest branch on Sun */

   ;/* move everything so that the boxcenter is in (0,0,0) */

   SoustractionVecteur(v0,triverts(0),*boxcenter)

   SoustractionVecteur(v1,triverts(1),*boxcenter)

   SoustractionVecteur(v2,triverts(2),*boxcenter)


   ;/* compute triangle edges */

   SoustractionVecteur(e0,v1,v0);      /* tri edge 0 */

   SoustractionVecteur(e1,v2,v1);      /* tri edge 1 */

   SoustractionVecteur(e2,v0,v2);      /* tri edge 2 */


   ;/* Bullet 3:  */

   ;/*  test the 9 tests first (this was faster) */

   fex = Abs(e0\x)

   fey = Abs(e0\y)

   fez = Abs(e0\z)

   AXISTEST_X01(e0\z, e0\y, fez, fey)

   AXISTEST_Y02(e0\z, e0\x, fez, fex)

   AXISTEST_Z12(e0\y, e0\x, fey, fex)



   fex = Abs(e1\x)

   fey = Abs(e1\y);

   fez = Abs(e1\z);

   AXISTEST_X01(e1\z, e1\y, fez, fey)

   AXISTEST_Y02(e1\z, e1\x, fez, fex)

   AXISTEST_Z0(e1\y, e1\x, fey, fex)

   fex = Abs(e2\x);

   fey = Abs(e2\y);

   fez = Abs(e2\z);

   AXISTEST_X2(e2\z, e2\y, fez, fey)

   AXISTEST_Y1(e2\z, e2\x, fez, fex)

   AXISTEST_Z12(e2\y, e2\x, fey, fex)


   ;/* Bullet 1: */

   ;/*  first test overlap in the {x,y,z}-directions */

   ;/*  find min, max of the triangle each direction, And test For overlap in */

   ;/*  that direction -- this is equivalent To testing a minimal AABB around */

   ;/*  the triangle against the AABB */



   ;/* test in X-direction */

   FINDMINMAX(v0\x,v1\x,v2\x,min,max)

   If min>*boxhalfsize\x Or max<-*boxhalfsize\x : ProcedureReturn #False : EndIf


   ;/* test in Y-direction */

   FINDMINMAX(v0\y,v1\y,v2\y,min,max)

   If min>*boxhalfsize\y Or max<-*boxhalfsize\y : ProcedureReturn #False : EndIf

   ;/* test in Z-direction */

   FINDMINMAX(v0\z,v1\z,v2\z,min,max)

   If min>*boxhalfsize\z Or max<-*boxhalfsize\z : ProcedureReturn #False : EndIf


   ;/* Bullet 2: */

   ;/*  test If the box intersects the plane of the triangle */

   ;/*  compute plane equation of triangle: normal*x+d=0 */

   ProduitVectoriel(normal,e0,e1)

   ;// -NJMP- (line removed here)

   If planeBoxOverlap(@normal,@v0,*boxhalfsize)=0 : ProcedureReturn #False : EndIf

   ProcedureReturn #True ;   /* box and triangle overlaps */

EndProcedure


; IDE Options = PureBasic v4.00 - Beta 6 (Windows - x86)
; CursorPosition = 268
; FirstLine = 268
; Folding = --